
#ifndef __nanojit_portability_h__
#define __nanojit_portability_h__

#include "AvmPlus.h"

#if defined AVMPLUS_IA32
#define NANOJIT_IA32
#elif defined AVMPLUS_ARM
#if defined(TARGET_THUMB2) || defined(UNDER_RT)
#define NANOJIT_THUMB2
#else
#define NANOJIT_ARM
#endif
#elif defined AVMPLUS_PPC
#define NANOJIT_PPC
#elif defined AVMPLUS_SPARC
#define NANOJIT_SPARC
#elif defined AVMPLUS_AMD64
#define NANOJIT_X64
#elif defined VMCFG_SH4
#define NANOJIT_SH4
#elif defined AVMPLUS_MIPS
#define NANOJIT_MIPS
#else
#error "unknown nanojit architecture"
#endif


#ifdef AVMPLUS_64BIT
#define NANOJIT_64BIT
#endif

#if defined NANOJIT_64BIT
#define IF_64BIT(...) __VA_ARGS__
#define UNLESS_64BIT(...)
#define CASE32(x)
#define CASE64(x)   case x
#else
#define IF_64BIT(...)
#define UNLESS_64BIT(...) __VA_ARGS__
#define CASE32(x)   case x
#define CASE64(x)
#endif

#if defined NANOJIT_IA32 || defined NANOJIT_X64
#define CASE86(x)   case x
#else
#define CASE86(x)
#endif

// Embed no-op macros that let Valgrind work with the JIT.
#ifdef MOZ_VALGRIND
#  define JS_VALGRIND
#endif
#ifdef JS_VALGRIND
#  include <valgrind/valgrind.h>
#elif !defined(VALGRIND_DISCARD_TRANSLATIONS)
#  define VALGRIND_DISCARD_TRANSLATIONS(addr, szB)
#endif

#if defined(__ghs__)
#define NANOJIT_FORCEINLINE __attribute__((always_inline))
#else
#define NANOJIT_FORCEINLINE inline
#endif


#endif // #ifndef __nanojit_portability_h__

